# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# NOTE! THIS FILE IS AUTOMATICALLY GENERATED AND WILL BE
# OVERWRITTEN WHEN PREPARING PACKAGES.
#
# IF YOU WANT TO MODIFY THIS FILE, YOU SHOULD MODIFY THE TEMPLATE
# `get_provider_info_TEMPLATE.py.jinja2` IN the `dev/breeze/src/airflow_breeze/templates` DIRECTORY


def get_provider_info():
    return {
        "package-name": "apache-airflow-providers-google",
        "name": "Google",
        "description": "Google services including:\n\n  - `Google Ads <https://ads.google.com/>`__\n  - `Google Cloud (GCP) <https://cloud.google.com/>`__\n  - `Google Firebase <https://firebase.google.com/>`__\n  - `Google LevelDB <https://github.com/google/leveldb/>`__\n  - `Google Marketing Platform <https://marketingplatform.google.com/>`__\n  - `Google Workspace <https://workspace.google.com/>`__ (formerly Google Suite)\n",
        "state": "ready",
        "source-date-epoch": 1723970253,
        "versions": [
            "10.22.0",
            "10.21.1",
            "10.21.0",
            "10.20.0",
            "10.19.0",
            "10.18.0",
            "10.17.0",
            "10.16.0",
            "10.15.0",
            "10.14.0",
            "10.13.1",
            "10.13.0",
            "10.12.0",
            "10.11.1",
            "10.11.0",
            "10.10.1",
            "10.10.0",
            "10.9.0",
            "10.8.0",
            "10.7.0",
            "10.6.0",
            "10.5.0",
            "10.4.0",
            "10.3.0",
            "10.2.0",
            "10.1.1",
            "10.1.0",
            "10.0.0",
            "9.0.0",
            "8.12.0",
            "8.11.0",
            "8.10.0",
            "8.9.0",
            "8.8.0",
            "8.7.0",
            "8.6.0",
            "8.5.0",
            "8.4.0",
            "8.3.0",
            "8.2.0",
            "8.1.0",
            "8.0.0",
            "7.0.0",
            "6.8.0",
            "6.7.0",
            "6.6.0",
            "6.5.0",
            "6.4.0",
            "6.3.0",
            "6.2.0",
            "6.1.0",
            "6.0.0",
            "5.1.0",
            "5.0.0",
            "4.0.0",
            "3.0.0",
            "2.2.0",
            "2.1.0",
            "2.0.0",
            "1.0.0",
        ],
        "dependencies": [
            "apache-airflow>=2.8.0",
            "apache-airflow-providers-common-compat>=1.1.0",
            "apache-airflow-providers-common-sql>=1.7.2",
            "asgiref>=3.5.2",
            "dill>=0.2.3",
            "gcloud-aio-auth>=5.2.0",
            "gcloud-aio-bigquery>=6.1.2",
            "gcloud-aio-storage>=9.0.0",
            "gcsfs>=2023.10.0",
            "google-ads>=24.1.0",
            "google-analytics-admin>=0.9.0",
            "google-api-core>=2.11.0,!=2.16.0,!=2.18.0",
            "google-api-python-client>=2.0.2",
            "google-auth>=2.29.0",
            "google-auth-httplib2>=0.0.1",
            "google-cloud-aiplatform>=1.57.0",
            "google-cloud-automl>=2.12.0",
            "google-cloud-bigquery<3.21.0,>=3.4.0",
            "google-cloud-bigquery-datatransfer>=3.13.0",
            "google-cloud-bigtable>=2.17.0",
            "google-cloud-build>=3.22.0",
            "google-cloud-compute>=1.10.0",
            "google-cloud-container>=2.17.4",
            "google-cloud-datacatalog>=3.11.1",
            "google-cloud-dataflow-client>=0.8.6",
            "google-cloud-dataform>=0.5.0",
            "google-cloud-dataplex>=1.10.0",
            "google-cloud-dataproc>=5.8.0",
            "google-cloud-dataproc-metastore>=1.12.0",
            "google-cloud-dlp>=3.12.0",
            "google-cloud-kms>=2.15.0",
            "google-cloud-language>=2.9.0",
            "google-cloud-logging>=3.5.0",
            "google-cloud-memcache>=1.7.0",
            "google-cloud-monitoring>=2.18.0",
            "google-cloud-orchestration-airflow>=1.10.0",
            "google-cloud-os-login>=2.9.1",
            "google-cloud-pubsub>=2.19.0",
            "google-cloud-redis>=2.12.0",
            "google-cloud-secret-manager>=2.16.0",
            "google-cloud-spanner>=3.11.1",
            "google-cloud-speech>=2.18.0",
            "google-cloud-storage>=2.7.0",
            "google-cloud-storage-transfer>=1.4.1",
            "google-cloud-tasks>=2.13.0",
            "google-cloud-texttospeech>=2.14.1",
            "google-cloud-translate>=3.11.0",
            "google-cloud-videointelligence>=2.11.0",
            "google-cloud-vision>=3.4.0",
            "google-cloud-workflows>=1.10.0",
            "google-cloud-run>=0.10.0",
            "google-cloud-batch>=0.13.0",
            "grpcio-gcp>=0.2.2",
            "httpx>=0.25.0",
            "json-merge-patch>=0.2",
            "looker-sdk>=22.4.0",
            "pandas-gbq>=0.7.0",
            'pandas>=2.1.2,<2.2;python_version>="3.9"',
            'pandas>=1.5.3,<2.2;python_version<"3.9"',
            "proto-plus>=1.19.6",
            "python-slugify>=7.0.0",
            "PyOpenSSL>=23.0.0",
            "sqlalchemy-bigquery>=1.2.1",
            "sqlalchemy-spanner>=1.6.2",
            "tenacity>=8.1.0",
        ],
        "additional-extras": [
            {"name": "apache.beam", "dependencies": ["apache-beam[gcp]"]},
            {"name": "cncf.kubernetes", "dependencies": ["apache-airflow-providers-cncf-kubernetes>=7.2.0"]},
            {"name": "leveldb", "dependencies": ["plyvel"]},
            {"name": "oracle", "dependencies": ["apache-airflow-providers-oracle>=3.1.0"]},
            {"name": "facebook", "dependencies": ["apache-airflow-providers-facebook>=2.2.0"]},
            {"name": "amazon", "dependencies": ["apache-airflow-providers-amazon>=2.6.0"]},
        ],
        "integrations": [
            {
                "integration-name": "Google Analytics (GA4)",
                "external-doc-url": "https://analytics.google.com/",
                "logo": "/integration-logos/gcp/Google-Analytics.png",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/marketing_platform/analytics_admin.rst"
                ],
                "tags": ["gmp"],
            },
            {
                "integration-name": "Google Analytics360",
                "external-doc-url": "https://analytics.google.com/",
                "logo": "/integration-logos/gcp/Google-Analytics.png",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/marketing_platform/analytics.rst"
                ],
                "tags": ["gmp"],
            },
            {
                "integration-name": "Google Ads",
                "external-doc-url": "https://ads.google.com/",
                "logo": "/integration-logos/gcp/Google-Ads.png",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/ads.rst"],
                "tags": ["gmp"],
            },
            {
                "integration-name": "Google AutoML",
                "external-doc-url": "https://cloud.google.com/automl/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/automl.rst"],
                "logo": "/integration-logos/gcp/Cloud-AutoML.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google BigQuery Data Transfer Service",
                "external-doc-url": "https://cloud.google.com/bigquery/transfer/",
                "logo": "/integration-logos/gcp/BigQuery.png",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/bigquery_dts.rst"],
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google BigQuery",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/bigquery.rst"],
                "external-doc-url": "https://cloud.google.com/bigquery/",
                "logo": "/integration-logos/gcp/BigQuery.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Bigtable",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/bigtable.rst"],
                "external-doc-url": "https://cloud.google.com/bigtable/",
                "logo": "/integration-logos/gcp/Cloud-Bigtable.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Build",
                "external-doc-url": "https://cloud.google.com/build/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/cloud_build.rst"],
                "logo": "/integration-logos/gcp/Cloud-Build.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Common",
                "external-doc-url": "https://cloud.google.com/",
                "tags": ["gcp"],
                "logo": "/integration-logos/gcp/Google.png",
            },
            {
                "integration-name": "Google Cloud Composer",
                "external-doc-url": "https://cloud.google.com/composer",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/cloud_composer.rst"],
                "tags": ["google"],
            },
            {
                "integration-name": "Google Cloud Run",
                "external-doc-url": "https://cloud.google.com/run",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/cloud_run.rst"],
                "tags": ["google"],
            },
            {
                "integration-name": "Google Cloud Batch",
                "external-doc-url": "https://cloud.google.com/batch",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/cloud_batch.rst"],
                "tags": ["google"],
            },
            {
                "integration-name": "Google Cloud Dataform",
                "external-doc-url": "https://cloud.google.com/dataform/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/dataform.rst"],
                "tags": ["google"],
            },
            {
                "integration-name": "Google Cloud Data Loss Prevention (DLP)",
                "external-doc-url": "https://cloud.google.com/dlp/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/cloud/data_loss_prevention.rst"
                ],
                "logo": "/integration-logos/gcp/google-data-loss-prevention.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Firestore",
                "external-doc-url": "https://firebase.google.com/docs/firestore",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/firebase/firestore.rst"],
                "logo": "/integration-logos/gcp/Google-Firestore.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Functions",
                "external-doc-url": "https://cloud.google.com/functions/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/functions.rst"],
                "logo": "/integration-logos/gcp/Cloud-Functions.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Key Management Service (KMS)",
                "external-doc-url": "https://cloud.google.com/security-key-management/",
                "logo": "/integration-logos/gcp/Key-Management-Service.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Life Sciences",
                "external-doc-url": "https://cloud.google.com/life-sciences/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/life_sciences.rst"],
                "logo": "/integration-logos/gcp/Google-Cloud-Life-Sciences.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Memorystore",
                "external-doc-url": "https://cloud.google.com/memorystore/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/cloud/cloud_memorystore.rst",
                    "/docs/apache-airflow-providers-google/operators/cloud/cloud_memorystore_memcached.rst",
                ],
                "logo": "/integration-logos/gcp/Cloud-Memorystore.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud OS Login",
                "external-doc-url": "https://cloud.google.com/compute/docs/oslogin/",
                "logo": "/integration-logos/gcp/Google-Cloud-Generic.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Pub/Sub",
                "external-doc-url": "https://cloud.google.com/pubsub/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/pubsub.rst"],
                "logo": "/integration-logos/gcp/Cloud-PubSub.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Secret Manager",
                "external-doc-url": "https://cloud.google.com/secret-manager/",
                "logo": "/integration-logos/gcp/Google-Cloud-Secret-Manager.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Spanner",
                "external-doc-url": "https://cloud.google.com/spanner/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/spanner.rst"],
                "logo": "/integration-logos/gcp/Cloud-Spanner.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Speech-to-Text",
                "external-doc-url": "https://cloud.google.com/speech-to-text/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/cloud/speech_to_text.rst",
                    "/docs/apache-airflow-providers-google/operators/cloud/translate_speech.rst",
                ],
                "logo": "/integration-logos/gcp/Cloud-Speech-to-Text.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud SQL",
                "external-doc-url": "https://cloud.google.com/sql/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/cloud_sql.rst"],
                "logo": "/integration-logos/gcp/Cloud-SQL.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Stackdriver",
                "external-doc-url": "https://cloud.google.com/products/operations/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/stackdriver.rst"],
                "logo": "/integration-logos/gcp/Google-Cloud-Stackdriver.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Storage (GCS)",
                "external-doc-url": "https://cloud.google.com/storage/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/gcs.rst"],
                "logo": "/integration-logos/gcp/Cloud-Storage.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Tasks",
                "external-doc-url": "https://cloud.google.com/tasks/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/tasks.rst"],
                "logo": "/integration-logos/gcp/Cloud-Tasks.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Text-to-Speech",
                "external-doc-url": "https://cloud.google.com/text-to-speech/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/text_to_speech.rst"],
                "logo": "/integration-logos/gcp/Cloud-Text-to-Speech.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Translation",
                "external-doc-url": "https://cloud.google.com/translate/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/cloud/translate.rst",
                    "/docs/apache-airflow-providers-google/operators/cloud/translate_speech.rst",
                ],
                "logo": "/integration-logos/gcp/Cloud-Translation-API.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Video Intelligence",
                "external-doc-url": "https://cloud.google.com/video-intelligence/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/cloud/video_intelligence.rst"
                ],
                "logo": "/integration-logos/gcp/Cloud-Video-Intelligence-API.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Vision",
                "external-doc-url": "https://cloud.google.com/vision/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/vision.rst"],
                "logo": "/integration-logos/gcp/Cloud-Vision-API.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Compute Engine",
                "external-doc-url": "https://cloud.google.com/compute/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/cloud/compute.rst",
                    "/docs/apache-airflow-providers-google/operators/cloud/compute_ssh.rst",
                ],
                "logo": "/integration-logos/gcp/Compute-Engine.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Data Proc",
                "external-doc-url": "https://cloud.google.com/dataproc/",
                "logo": "/integration-logos/gcp/Google-Data-Proc.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Data Catalog",
                "external-doc-url": "https://cloud.google.com/data-catalog/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/datacatalog.rst"],
                "logo": "/integration-logos/gcp/Google-Data-Catalog.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Dataflow",
                "external-doc-url": "https://cloud.google.com/dataflow/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/dataflow.rst"],
                "logo": "/integration-logos/gcp/Cloud-Dataflow.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Data Pipelines",
                "external-doc-url": "https://cloud.google.com/dataflow/docs/reference/data-pipelines/rest",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/datapipeline.rst"],
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Data Fusion",
                "external-doc-url": "https://cloud.google.com/data-fusion/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/datafusion.rst"],
                "logo": "/integration-logos/gcp/Google-Data-Fusion.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Dataplex",
                "external-doc-url": "https://cloud.google.com/dataplex/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/dataplex.rst"],
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Dataprep",
                "external-doc-url": "https://cloud.google.com/dataprep/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/dataprep.rst"],
                "logo": "/integration-logos/gcp/Google-Dataprep.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Dataproc Metastore",
                "external-doc-url": "https://cloud.google.com/dataproc-metastore/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/cloud/dataproc_metastore.rst"
                ],
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Dataproc",
                "external-doc-url": "https://cloud.google.com/dataproc/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/dataproc.rst"],
                "logo": "/integration-logos/gcp/Cloud-Dataproc.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Datastore",
                "external-doc-url": "https://cloud.google.com/datastore/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/datastore.rst"],
                "logo": "/integration-logos/gcp/Cloud-Datastore.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Deployment Manager",
                "external-doc-url": "https://cloud.google.com/deployment-manager/",
                "logo": "/integration-logos/gcp/Google-Deployment-Manager.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google API Python Client",
                "external-doc-url": "https://github.com/googleapis/google-api-python-client/",
                "logo": "/integration-logos/gcp/Google-API-Python-Client.png",
                "tags": ["google"],
            },
            {
                "integration-name": "Google Calendar",
                "external-doc-url": "https://calendar.google.com/",
                "logo": "/integration-logos/gcp/Google-Calendar.png",
                "tags": ["google"],
            },
            {
                "integration-name": "Google Campaign Manager",
                "external-doc-url": "https://developers.google.com/doubleclick-advertisers",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/marketing_platform/campaign_manager.rst"
                ],
                "logo": "/integration-logos/gcp/Google-Campaign-Manager.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud",
                "external-doc-url": "https://cloud.google.com/",
                "logo": "/integration-logos/gcp/Google-Cloud.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Discovery API",
                "external-doc-url": "https://developers.google.com/discovery/",
                "logo": "/integration-logos/gcp/Google-Cloud-Generic.png",
                "tags": ["google"],
            },
            {
                "integration-name": "Google Display&Video 360",
                "external-doc-url": "https://marketingplatform.google.com/about/display-video-360/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/marketing_platform/display_video.rst"
                ],
                "logo": "/integration-logos/gcp/Google-Display-And-Video-360.png",
                "tags": ["gmp"],
            },
            {
                "integration-name": "Google Drive",
                "external-doc-url": "https://www.google.com/drive/",
                "logo": "/integration-logos/gcp/Google-Drive.png",
                "tags": ["google"],
            },
            {
                "integration-name": "Google Search Ads 360",
                "external-doc-url": "https://marketingplatform.google.com/about/search-ads-360/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/marketing_platform/search_ads.rst"
                ],
                "logo": "/integration-logos/gcp/Google-Search-Ads360.png",
                "tags": ["gmp"],
            },
            {
                "integration-name": "Google",
                "external-doc-url": "https://developer.google.com/",
                "logo": "/integration-logos/gcp/Google.png",
                "tags": ["google"],
            },
            {
                "integration-name": "Google Spreadsheet",
                "external-doc-url": "https://www.google.com/intl/en/sheets/about/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/suite/sheets.rst"],
                "logo": "/integration-logos/gcp/Google-Spreadsheet.png",
                "tags": ["google"],
            },
            {
                "integration-name": "Google Cloud Storage Transfer Service",
                "external-doc-url": "https://cloud.google.com/storage-transfer-service/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/cloud/cloud_storage_transfer_service.rst"
                ],
                "logo": "/integration-logos/gcp/Cloud-Storage.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Kubernetes Engine",
                "external-doc-url": "https://cloud.google.com/kubernetes-engine/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/cloud/kubernetes_engine.rst"
                ],
                "logo": "/integration-logos/gcp/Kubernetes-Engine.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Machine Learning Engine",
                "external-doc-url": "https://cloud.google.com/vertex-ai/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/mlengine.rst"],
                "logo": "/integration-logos/gcp/AI-Platform.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Natural Language",
                "external-doc-url": "https://cloud.google.com/natural-language/",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-google/operators/cloud/natural_language.rst"
                ],
                "logo": "/integration-logos/gcp/Cloud-NLP.png",
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Cloud Workflows",
                "external-doc-url": "https://cloud.google.com/workflows/",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/workflows.rst"],
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google LevelDB",
                "external-doc-url": "https://github.com/google/leveldb/blob/main/doc/index.md",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/leveldb/leveldb.rst"],
                "tags": ["google"],
            },
            {
                "integration-name": "Google Vertex AI",
                "external-doc-url": "https://cloud.google.com/vertex-ai",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/vertex_ai.rst"],
                "tags": ["gcp"],
            },
            {
                "integration-name": "Google Looker",
                "external-doc-url": "https://cloud.google.com/looker",
                "logo": "/integration-logos/gcp/Cloud-Looker.png",
                "how-to-guide": ["/docs/apache-airflow-providers-google/operators/cloud/looker.rst"],
                "tags": ["gcp"],
            },
        ],
        "operators": [
            {
                "integration-name": "Google Ads",
                "python-modules": ["airflow.providers.google.ads.operators.ads"],
            },
            {
                "integration-name": "Google Cloud Common",
                "python-modules": ["airflow.providers.google.cloud.operators.cloud_base"],
            },
            {
                "integration-name": "Google AutoML",
                "python-modules": ["airflow.providers.google.cloud.operators.automl"],
            },
            {
                "integration-name": "Google BigQuery",
                "python-modules": ["airflow.providers.google.cloud.operators.bigquery"],
            },
            {
                "integration-name": "Google BigQuery Data Transfer Service",
                "python-modules": ["airflow.providers.google.cloud.operators.bigquery_dts"],
            },
            {
                "integration-name": "Google Bigtable",
                "python-modules": ["airflow.providers.google.cloud.operators.bigtable"],
            },
            {
                "integration-name": "Google Cloud Build",
                "python-modules": ["airflow.providers.google.cloud.operators.cloud_build"],
            },
            {
                "integration-name": "Google Cloud Composer",
                "python-modules": ["airflow.providers.google.cloud.operators.cloud_composer"],
            },
            {
                "integration-name": "Google Cloud Run",
                "python-modules": ["airflow.providers.google.cloud.operators.cloud_run"],
            },
            {
                "integration-name": "Google Cloud Memorystore",
                "python-modules": ["airflow.providers.google.cloud.operators.cloud_memorystore"],
            },
            {
                "integration-name": "Google Cloud SQL",
                "python-modules": ["airflow.providers.google.cloud.operators.cloud_sql"],
            },
            {
                "integration-name": "Google Cloud Storage Transfer Service",
                "python-modules": ["airflow.providers.google.cloud.operators.cloud_storage_transfer_service"],
            },
            {
                "integration-name": "Google Compute Engine",
                "python-modules": ["airflow.providers.google.cloud.operators.compute"],
            },
            {
                "integration-name": "Google Data Catalog",
                "python-modules": ["airflow.providers.google.cloud.operators.datacatalog"],
            },
            {
                "integration-name": "Google Dataflow",
                "python-modules": ["airflow.providers.google.cloud.operators.dataflow"],
            },
            {
                "integration-name": "Google Data Pipelines",
                "python-modules": ["airflow.providers.google.cloud.operators.datapipeline"],
            },
            {
                "integration-name": "Google Data Fusion",
                "python-modules": ["airflow.providers.google.cloud.operators.datafusion"],
            },
            {
                "integration-name": "Google Dataplex",
                "python-modules": ["airflow.providers.google.cloud.operators.dataplex"],
            },
            {
                "integration-name": "Google Dataprep",
                "python-modules": ["airflow.providers.google.cloud.operators.dataprep"],
            },
            {
                "integration-name": "Google Dataproc Metastore",
                "python-modules": ["airflow.providers.google.cloud.operators.dataproc_metastore"],
            },
            {
                "integration-name": "Google Dataproc",
                "python-modules": ["airflow.providers.google.cloud.operators.dataproc"],
            },
            {
                "integration-name": "Google Datastore",
                "python-modules": ["airflow.providers.google.cloud.operators.datastore"],
            },
            {
                "integration-name": "Google Cloud Data Loss Prevention (DLP)",
                "python-modules": ["airflow.providers.google.cloud.operators.dlp"],
            },
            {
                "integration-name": "Google Cloud Functions",
                "python-modules": ["airflow.providers.google.cloud.operators.functions"],
            },
            {
                "integration-name": "Google Cloud Storage (GCS)",
                "python-modules": ["airflow.providers.google.cloud.operators.gcs"],
            },
            {
                "integration-name": "Google Kubernetes Engine",
                "python-modules": ["airflow.providers.google.cloud.operators.kubernetes_engine"],
            },
            {
                "integration-name": "Google Cloud Life Sciences",
                "python-modules": ["airflow.providers.google.cloud.operators.life_sciences"],
            },
            {
                "integration-name": "Google Machine Learning Engine",
                "python-modules": ["airflow.providers.google.cloud.operators.mlengine"],
            },
            {
                "integration-name": "Google Cloud Natural Language",
                "python-modules": ["airflow.providers.google.cloud.operators.natural_language"],
            },
            {
                "integration-name": "Google Cloud Pub/Sub",
                "python-modules": ["airflow.providers.google.cloud.operators.pubsub"],
            },
            {
                "integration-name": "Google Cloud Spanner",
                "python-modules": ["airflow.providers.google.cloud.operators.spanner"],
            },
            {
                "integration-name": "Google Cloud Speech-to-Text",
                "python-modules": ["airflow.providers.google.cloud.operators.speech_to_text"],
            },
            {
                "integration-name": "Google Cloud Stackdriver",
                "python-modules": ["airflow.providers.google.cloud.operators.stackdriver"],
            },
            {
                "integration-name": "Google Cloud Tasks",
                "python-modules": ["airflow.providers.google.cloud.operators.tasks"],
            },
            {
                "integration-name": "Google Cloud Text-to-Speech",
                "python-modules": [
                    "airflow.providers.google.cloud.operators.text_to_speech",
                    "airflow.providers.google.cloud.operators.translate_speech",
                ],
            },
            {
                "integration-name": "Google Cloud Translation",
                "python-modules": [
                    "airflow.providers.google.cloud.operators.translate",
                    "airflow.providers.google.cloud.operators.translate_speech",
                ],
            },
            {
                "integration-name": "Google Cloud Video Intelligence",
                "python-modules": ["airflow.providers.google.cloud.operators.video_intelligence"],
            },
            {
                "integration-name": "Google Cloud Vision",
                "python-modules": ["airflow.providers.google.cloud.operators.vision"],
            },
            {
                "integration-name": "Google Cloud Workflows",
                "python-modules": ["airflow.providers.google.cloud.operators.workflows"],
            },
            {
                "integration-name": "Google Cloud Firestore",
                "python-modules": ["airflow.providers.google.firebase.operators.firestore"],
            },
            {
                "integration-name": "Google Analytics (GA4)",
                "python-modules": ["airflow.providers.google.marketing_platform.operators.analytics_admin"],
            },
            {
                "integration-name": "Google Analytics360",
                "python-modules": ["airflow.providers.google.marketing_platform.operators.analytics"],
            },
            {
                "integration-name": "Google Campaign Manager",
                "python-modules": ["airflow.providers.google.marketing_platform.operators.campaign_manager"],
            },
            {
                "integration-name": "Google Display&Video 360",
                "python-modules": ["airflow.providers.google.marketing_platform.operators.display_video"],
            },
            {
                "integration-name": "Google Search Ads 360",
                "python-modules": ["airflow.providers.google.marketing_platform.operators.search_ads"],
            },
            {
                "integration-name": "Google Spreadsheet",
                "python-modules": ["airflow.providers.google.suite.operators.sheets"],
            },
            {
                "integration-name": "Google LevelDB",
                "python-modules": ["airflow.providers.google.leveldb.operators.leveldb"],
            },
            {
                "integration-name": "Google Vertex AI",
                "python-modules": [
                    "airflow.providers.google.cloud.operators.vertex_ai.dataset",
                    "airflow.providers.google.cloud.operators.vertex_ai.custom_job",
                    "airflow.providers.google.cloud.operators.vertex_ai.auto_ml",
                    "airflow.providers.google.cloud.operators.vertex_ai.batch_prediction_job",
                    "airflow.providers.google.cloud.operators.vertex_ai.endpoint_service",
                    "airflow.providers.google.cloud.operators.vertex_ai.hyperparameter_tuning_job",
                    "airflow.providers.google.cloud.operators.vertex_ai.model_service",
                    "airflow.providers.google.cloud.operators.vertex_ai.pipeline_job",
                    "airflow.providers.google.cloud.operators.vertex_ai.generative_model",
                ],
            },
            {
                "integration-name": "Google Looker",
                "python-modules": ["airflow.providers.google.cloud.operators.looker"],
            },
            {
                "integration-name": "Google Cloud Dataform",
                "python-modules": ["airflow.providers.google.cloud.operators.dataform"],
            },
            {
                "integration-name": "Google Cloud Batch",
                "python-modules": ["airflow.providers.google.cloud.operators.cloud_batch"],
            },
        ],
        "sensors": [
            {
                "integration-name": "Google BigQuery",
                "python-modules": ["airflow.providers.google.cloud.sensors.bigquery"],
            },
            {
                "integration-name": "Google BigQuery Data Transfer Service",
                "python-modules": ["airflow.providers.google.cloud.sensors.bigquery_dts"],
            },
            {
                "integration-name": "Google Bigtable",
                "python-modules": ["airflow.providers.google.cloud.sensors.bigtable"],
            },
            {
                "integration-name": "Google Cloud Composer",
                "python-modules": ["airflow.providers.google.cloud.sensors.cloud_composer"],
            },
            {
                "integration-name": "Google Cloud Storage Transfer Service",
                "python-modules": ["airflow.providers.google.cloud.sensors.cloud_storage_transfer_service"],
            },
            {
                "integration-name": "Google Dataflow",
                "python-modules": ["airflow.providers.google.cloud.sensors.dataflow"],
            },
            {
                "integration-name": "Google Data Fusion",
                "python-modules": ["airflow.providers.google.cloud.sensors.datafusion"],
            },
            {
                "integration-name": "Google Dataprep",
                "python-modules": ["airflow.providers.google.cloud.sensors.dataprep"],
            },
            {
                "integration-name": "Google Dataplex",
                "python-modules": ["airflow.providers.google.cloud.sensors.dataplex"],
            },
            {
                "integration-name": "Google Dataproc",
                "python-modules": ["airflow.providers.google.cloud.sensors.dataproc"],
            },
            {
                "integration-name": "Google Dataproc Metastore",
                "python-modules": ["airflow.providers.google.cloud.sensors.dataproc_metastore"],
            },
            {
                "integration-name": "Google Cloud Storage (GCS)",
                "python-modules": ["airflow.providers.google.cloud.sensors.gcs"],
            },
            {
                "integration-name": "Google Cloud Pub/Sub",
                "python-modules": ["airflow.providers.google.cloud.sensors.pubsub"],
            },
            {
                "integration-name": "Google Cloud Workflows",
                "python-modules": ["airflow.providers.google.cloud.sensors.workflows"],
            },
            {
                "integration-name": "Google Drive",
                "python-modules": ["airflow.providers.google.suite.sensors.drive"],
            },
            {
                "integration-name": "Google Campaign Manager",
                "python-modules": ["airflow.providers.google.marketing_platform.sensors.campaign_manager"],
            },
            {
                "integration-name": "Google Display&Video 360",
                "python-modules": ["airflow.providers.google.marketing_platform.sensors.display_video"],
            },
            {
                "integration-name": "Google Search Ads 360",
                "python-modules": ["airflow.providers.google.marketing_platform.sensors.search_ads"],
            },
            {
                "integration-name": "Google Looker",
                "python-modules": ["airflow.providers.google.cloud.sensors.looker"],
            },
            {
                "integration-name": "Google Cloud Dataform",
                "python-modules": ["airflow.providers.google.cloud.sensors.dataform"],
            },
            {
                "integration-name": "Google Cloud Tasks",
                "python-modules": ["airflow.providers.google.cloud.sensors.tasks"],
            },
        ],
        "filesystems": ["airflow.providers.google.cloud.fs.gcs"],
        "dataset-uris": [
            {"schemes": ["gcp"], "handler": None},
            {"schemes": ["bigquery"], "handler": "airflow.providers.google.datasets.bigquery.sanitize_uri"},
        ],
        "hooks": [
            {"integration-name": "Google Ads", "python-modules": ["airflow.providers.google.ads.hooks.ads"]},
            {
                "integration-name": "Google AutoML",
                "python-modules": ["airflow.providers.google.cloud.hooks.automl"],
            },
            {
                "integration-name": "Google BigQuery",
                "python-modules": ["airflow.providers.google.cloud.hooks.bigquery"],
            },
            {
                "integration-name": "Google BigQuery Data Transfer Service",
                "python-modules": ["airflow.providers.google.cloud.hooks.bigquery_dts"],
            },
            {
                "integration-name": "Google Bigtable",
                "python-modules": ["airflow.providers.google.cloud.hooks.bigtable"],
            },
            {
                "integration-name": "Google Cloud Build",
                "python-modules": ["airflow.providers.google.cloud.hooks.cloud_build"],
            },
            {
                "integration-name": "Google Cloud Composer",
                "python-modules": ["airflow.providers.google.cloud.hooks.cloud_composer"],
            },
            {
                "integration-name": "Google Cloud Run",
                "python-modules": ["airflow.providers.google.cloud.hooks.cloud_run"],
            },
            {
                "integration-name": "Google Cloud Memorystore",
                "python-modules": ["airflow.providers.google.cloud.hooks.cloud_memorystore"],
            },
            {
                "integration-name": "Google Cloud SQL",
                "python-modules": ["airflow.providers.google.cloud.hooks.cloud_sql"],
            },
            {
                "integration-name": "Google Cloud Storage Transfer Service",
                "python-modules": ["airflow.providers.google.cloud.hooks.cloud_storage_transfer_service"],
            },
            {
                "integration-name": "Google Compute Engine",
                "python-modules": [
                    "airflow.providers.google.cloud.hooks.compute",
                    "airflow.providers.google.cloud.hooks.compute_ssh",
                ],
            },
            {
                "integration-name": "Google Data Catalog",
                "python-modules": ["airflow.providers.google.cloud.hooks.datacatalog"],
            },
            {
                "integration-name": "Google Dataflow",
                "python-modules": ["airflow.providers.google.cloud.hooks.dataflow"],
            },
            {
                "integration-name": "Google Data Pipelines",
                "python-modules": ["airflow.providers.google.cloud.hooks.datapipeline"],
            },
            {
                "integration-name": "Google Data Fusion",
                "python-modules": ["airflow.providers.google.cloud.hooks.datafusion"],
            },
            {
                "integration-name": "Google Dataplex",
                "python-modules": ["airflow.providers.google.cloud.hooks.dataplex"],
            },
            {
                "integration-name": "Google Dataprep",
                "python-modules": ["airflow.providers.google.cloud.hooks.dataprep"],
            },
            {
                "integration-name": "Google Dataproc Metastore",
                "python-modules": ["airflow.providers.google.cloud.hooks.dataproc_metastore"],
            },
            {
                "integration-name": "Google Dataproc",
                "python-modules": ["airflow.providers.google.cloud.hooks.dataproc"],
            },
            {
                "integration-name": "Google Datastore",
                "python-modules": ["airflow.providers.google.cloud.hooks.datastore"],
            },
            {
                "integration-name": "Google Cloud Data Loss Prevention (DLP)",
                "python-modules": ["airflow.providers.google.cloud.hooks.dlp"],
            },
            {
                "integration-name": "Google Cloud Functions",
                "python-modules": ["airflow.providers.google.cloud.hooks.functions"],
            },
            {
                "integration-name": "Google Cloud Storage (GCS)",
                "python-modules": ["airflow.providers.google.cloud.hooks.gcs"],
            },
            {
                "integration-name": "Google Deployment Manager",
                "python-modules": ["airflow.providers.google.cloud.hooks.gdm"],
            },
            {
                "integration-name": "Google Cloud Key Management Service (KMS)",
                "python-modules": ["airflow.providers.google.cloud.hooks.kms"],
            },
            {
                "integration-name": "Google Kubernetes Engine",
                "python-modules": ["airflow.providers.google.cloud.hooks.kubernetes_engine"],
            },
            {
                "integration-name": "Google Cloud Life Sciences",
                "python-modules": ["airflow.providers.google.cloud.hooks.life_sciences"],
            },
            {
                "integration-name": "Google Machine Learning Engine",
                "python-modules": ["airflow.providers.google.cloud.hooks.mlengine"],
            },
            {
                "integration-name": "Google Cloud Natural Language",
                "python-modules": ["airflow.providers.google.cloud.hooks.natural_language"],
            },
            {
                "integration-name": "Google Cloud OS Login",
                "python-modules": ["airflow.providers.google.cloud.hooks.os_login"],
            },
            {
                "integration-name": "Google Cloud Pub/Sub",
                "python-modules": ["airflow.providers.google.cloud.hooks.pubsub"],
            },
            {
                "integration-name": "Google Cloud Secret Manager",
                "python-modules": ["airflow.providers.google.cloud.hooks.secret_manager"],
            },
            {
                "integration-name": "Google Cloud Spanner",
                "python-modules": ["airflow.providers.google.cloud.hooks.spanner"],
            },
            {
                "integration-name": "Google Cloud Speech-to-Text",
                "python-modules": ["airflow.providers.google.cloud.hooks.speech_to_text"],
            },
            {
                "integration-name": "Google Cloud Stackdriver",
                "python-modules": ["airflow.providers.google.cloud.hooks.stackdriver"],
            },
            {
                "integration-name": "Google Cloud Tasks",
                "python-modules": ["airflow.providers.google.cloud.hooks.tasks"],
            },
            {
                "integration-name": "Google Cloud Text-to-Speech",
                "python-modules": ["airflow.providers.google.cloud.hooks.text_to_speech"],
            },
            {
                "integration-name": "Google Cloud Translation",
                "python-modules": ["airflow.providers.google.cloud.hooks.translate"],
            },
            {
                "integration-name": "Google Cloud Video Intelligence",
                "python-modules": ["airflow.providers.google.cloud.hooks.video_intelligence"],
            },
            {
                "integration-name": "Google Cloud Vision",
                "python-modules": ["airflow.providers.google.cloud.hooks.vision"],
            },
            {
                "integration-name": "Google Cloud Workflows",
                "python-modules": ["airflow.providers.google.cloud.hooks.workflows"],
            },
            {
                "integration-name": "Google",
                "python-modules": ["airflow.providers.google.common.hooks.base_google"],
            },
            {
                "integration-name": "Google Discovery API",
                "python-modules": ["airflow.providers.google.common.hooks.discovery_api"],
            },
            {
                "integration-name": "Google Cloud Firestore",
                "python-modules": ["airflow.providers.google.firebase.hooks.firestore"],
            },
            {
                "integration-name": "Google Analytics (GA4)",
                "python-modules": ["airflow.providers.google.marketing_platform.hooks.analytics_admin"],
            },
            {
                "integration-name": "Google Analytics360",
                "python-modules": ["airflow.providers.google.marketing_platform.hooks.analytics"],
            },
            {
                "integration-name": "Google Campaign Manager",
                "python-modules": ["airflow.providers.google.marketing_platform.hooks.campaign_manager"],
            },
            {
                "integration-name": "Google Display&Video 360",
                "python-modules": ["airflow.providers.google.marketing_platform.hooks.display_video"],
            },
            {
                "integration-name": "Google Search Ads 360",
                "python-modules": ["airflow.providers.google.marketing_platform.hooks.search_ads"],
            },
            {
                "integration-name": "Google Calendar",
                "python-modules": ["airflow.providers.google.suite.hooks.calendar"],
            },
            {
                "integration-name": "Google Drive",
                "python-modules": ["airflow.providers.google.suite.hooks.drive"],
            },
            {
                "integration-name": "Google Spreadsheet",
                "python-modules": ["airflow.providers.google.suite.hooks.sheets"],
            },
            {
                "integration-name": "Google LevelDB",
                "python-modules": ["airflow.providers.google.leveldb.hooks.leveldb"],
            },
            {
                "integration-name": "Google Vertex AI",
                "python-modules": [
                    "airflow.providers.google.cloud.hooks.vertex_ai.dataset",
                    "airflow.providers.google.cloud.hooks.vertex_ai.custom_job",
                    "airflow.providers.google.cloud.hooks.vertex_ai.auto_ml",
                    "airflow.providers.google.cloud.hooks.vertex_ai.batch_prediction_job",
                    "airflow.providers.google.cloud.hooks.vertex_ai.endpoint_service",
                    "airflow.providers.google.cloud.hooks.vertex_ai.hyperparameter_tuning_job",
                    "airflow.providers.google.cloud.hooks.vertex_ai.model_service",
                    "airflow.providers.google.cloud.hooks.vertex_ai.pipeline_job",
                    "airflow.providers.google.cloud.hooks.vertex_ai.generative_model",
                    "airflow.providers.google.cloud.hooks.vertex_ai.prediction_service",
                ],
            },
            {
                "integration-name": "Google Looker",
                "python-modules": ["airflow.providers.google.cloud.hooks.looker"],
            },
            {
                "integration-name": "Google Cloud Dataform",
                "python-modules": ["airflow.providers.google.cloud.hooks.dataform"],
            },
            {
                "integration-name": "Google Cloud Batch",
                "python-modules": ["airflow.providers.google.cloud.hooks.cloud_batch"],
            },
        ],
        "triggers": [
            {
                "integration-name": "Google BigQuery Data Transfer Service",
                "python-modules": ["airflow.providers.google.cloud.triggers.bigquery_dts"],
            },
            {
                "integration-name": "Google BigQuery",
                "python-modules": ["airflow.providers.google.cloud.triggers.bigquery"],
            },
            {
                "integration-name": "Google Cloud Build",
                "python-modules": ["airflow.providers.google.cloud.triggers.cloud_build"],
            },
            {
                "integration-name": "Google Cloud Composer",
                "python-modules": ["airflow.providers.google.cloud.triggers.cloud_composer"],
            },
            {
                "integration-name": "Google Cloud Run",
                "python-modules": ["airflow.providers.google.cloud.triggers.cloud_run"],
            },
            {
                "integration-name": "Google Cloud Storage Transfer Service",
                "python-modules": ["airflow.providers.google.cloud.triggers.cloud_storage_transfer_service"],
            },
            {
                "integration-name": "Google Cloud SQL",
                "python-modules": ["airflow.providers.google.cloud.triggers.cloud_sql"],
            },
            {
                "integration-name": "Google Dataflow",
                "python-modules": ["airflow.providers.google.cloud.triggers.dataflow"],
            },
            {
                "integration-name": "Google Data Fusion",
                "python-modules": ["airflow.providers.google.cloud.triggers.datafusion"],
            },
            {
                "integration-name": "Google Dataplex",
                "python-modules": ["airflow.providers.google.cloud.triggers.dataplex"],
            },
            {
                "integration-name": "Google Dataproc",
                "python-modules": ["airflow.providers.google.cloud.triggers.dataproc"],
            },
            {
                "integration-name": "Google Cloud Storage (GCS)",
                "python-modules": ["airflow.providers.google.cloud.triggers.gcs"],
            },
            {
                "integration-name": "Google Kubernetes Engine",
                "python-modules": ["airflow.providers.google.cloud.triggers.kubernetes_engine"],
            },
            {
                "integration-name": "Google Machine Learning Engine",
                "python-modules": ["airflow.providers.google.cloud.triggers.mlengine"],
            },
            {
                "integration-name": "Google Cloud Pub/Sub",
                "python-modules": ["airflow.providers.google.cloud.triggers.pubsub"],
            },
            {
                "integration-name": "Google Cloud",
                "python-modules": ["airflow.providers.google.cloud.triggers.cloud_batch"],
            },
            {
                "integration-name": "Google Vertex AI",
                "python-modules": ["airflow.providers.google.cloud.triggers.vertex_ai"],
            },
        ],
        "transfers": [
            {
                "source-integration-name": "Presto",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/presto_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.presto_to_gcs",
            },
            {
                "source-integration-name": "Trino",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/trino_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.trino_to_gcs",
            },
            {
                "source-integration-name": "Common SQL",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "python-module": "airflow.providers.google.cloud.transfers.sql_to_gcs",
            },
            {
                "source-integration-name": "Google Cloud Storage (GCS)",
                "target-integration-name": "Google Drive",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/gcs_to_gdrive.rst",
                "python-module": "airflow.providers.google.suite.transfers.gcs_to_gdrive",
            },
            {
                "source-integration-name": "Google Drive",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/gdrive_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.gdrive_to_gcs",
            },
            {
                "source-integration-name": "Microsoft Azure FileShare",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/azure_fileshare_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.azure_fileshare_to_gcs",
            },
            {
                "source-integration-name": "Apache Cassandra",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "python-module": "airflow.providers.google.cloud.transfers.cassandra_to_gcs",
            },
            {
                "source-integration-name": "Google Calendar",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/calendar_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.calendar_to_gcs",
            },
            {
                "source-integration-name": "Google Spreadsheet",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/sheets_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.sheets_to_gcs",
            },
            {
                "source-integration-name": "Amazon Simple Storage Service (S3)",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/s3_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.s3_to_gcs",
            },
            {
                "source-integration-name": "Google Cloud Storage (GCS)",
                "target-integration-name": "SSH File Transfer Protocol (SFTP)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/gcs_to_sftp.rst",
                "python-module": "airflow.providers.google.cloud.transfers.gcs_to_sftp",
            },
            {
                "source-integration-name": "PostgreSQL",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/postgres_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.postgres_to_gcs",
            },
            {
                "source-integration-name": "Google BigQuery",
                "target-integration-name": "Common SQL",
                "python-module": "airflow.providers.google.cloud.transfers.bigquery_to_sql",
            },
            {
                "source-integration-name": "Google BigQuery",
                "target-integration-name": "MySQL",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/bigquery_to_mysql.rst",
                "python-module": "airflow.providers.google.cloud.transfers.bigquery_to_mysql",
            },
            {
                "source-integration-name": "Google BigQuery",
                "target-integration-name": "Microsoft SQL Server (MSSQL)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/bigquery_to_mssql.rst",
                "python-module": "airflow.providers.google.cloud.transfers.bigquery_to_mssql",
            },
            {
                "source-integration-name": "Google BigQuery",
                "target-integration-name": "PostgreSQL",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/bigquery_to_postgres.rst",
                "python-module": "airflow.providers.google.cloud.transfers.bigquery_to_postgres",
            },
            {
                "source-integration-name": "Google Cloud Storage (GCS)",
                "target-integration-name": "Google BigQuery",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/gcs_to_bigquery.rst",
                "python-module": "airflow.providers.google.cloud.transfers.gcs_to_bigquery",
            },
            {
                "source-integration-name": "Google Cloud Storage (GCS)",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/gcs_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.gcs_to_gcs",
            },
            {
                "source-integration-name": "Facebook Ads",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/facebook_ads_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.facebook_ads_to_gcs",
            },
            {
                "source-integration-name": "SSH File Transfer Protocol (SFTP)",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/sftp_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.sftp_to_gcs",
            },
            {
                "source-integration-name": "Microsoft Azure Data Lake Storage",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "python-module": "airflow.providers.google.cloud.transfers.adls_to_gcs",
            },
            {
                "source-integration-name": "Google BigQuery",
                "target-integration-name": "Google BigQuery",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/bigquery_to_bigquery.rst",
                "python-module": "airflow.providers.google.cloud.transfers.bigquery_to_bigquery",
            },
            {
                "source-integration-name": "MySQL",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "python-module": "airflow.providers.google.cloud.transfers.mysql_to_gcs",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/mysql_to_gcs.rst",
            },
            {
                "source-integration-name": "Oracle",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "python-module": "airflow.providers.google.cloud.transfers.oracle_to_gcs",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/oracle_to_gcs.rst",
            },
            {
                "source-integration-name": "Google Cloud Storage (GCS)",
                "target-integration-name": "Google Spreadsheet",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/gcs_to_sheets.rst",
                "python-module": "airflow.providers.google.suite.transfers.gcs_to_sheets",
            },
            {
                "source-integration-name": "Common SQL",
                "target-integration-name": "Google Spreadsheet",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/sql_to_sheets.rst",
                "python-module": "airflow.providers.google.suite.transfers.sql_to_sheets",
            },
            {
                "source-integration-name": "Local",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/local_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.local_to_gcs",
            },
            {
                "source-integration-name": "Google BigQuery",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/bigquery_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.bigquery_to_gcs",
            },
            {
                "source-integration-name": "Google Cloud Storage (GCS)",
                "target-integration-name": "Local",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/gcs_to_local.rst",
                "python-module": "airflow.providers.google.cloud.transfers.gcs_to_local",
            },
            {
                "source-integration-name": "Google Drive",
                "target-integration-name": "Local",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/gdrive_to_local.rst",
                "python-module": "airflow.providers.google.cloud.transfers.gdrive_to_local",
            },
            {
                "source-integration-name": "Local",
                "target-integration-name": "Google Drive",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/local_to_drive.rst",
                "python-module": "airflow.providers.google.suite.transfers.local_to_drive",
            },
            {
                "source-integration-name": "Salesforce",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/salesforce_to_gcs.rst",
                "python-module": "airflow.providers.google.cloud.transfers.salesforce_to_gcs",
            },
            {
                "source-integration-name": "Google Ads",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "python-module": "airflow.providers.google.ads.transfers.ads_to_gcs",
            },
            {
                "source-integration-name": "Microsoft SQL Server (MSSQL)",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "python-module": "airflow.providers.google.cloud.transfers.mssql_to_gcs",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/mssql_to_gcs.rst",
            },
            {
                "source-integration-name": "Microsoft Azure Blob Storage",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "python-module": "airflow.providers.google.cloud.transfers.azure_blob_to_gcs",
                "how-to-guide": "/docs/apache-airflow-providers-google/operators/transfer/azure_blob_to_gcs.rst",
            },
        ],
        "connection-types": [
            {
                "hook-class-name": "airflow.providers.google.common.hooks.base_google.GoogleBaseHook",
                "connection-type": "google_cloud_platform",
            },
            {
                "hook-class-name": "airflow.providers.google.cloud.hooks.dataprep.GoogleDataprepHook",
                "connection-type": "dataprep",
            },
            {
                "hook-class-name": "airflow.providers.google.cloud.hooks.cloud_sql.CloudSQLHook",
                "connection-type": "gcpcloudsql",
            },
            {
                "hook-class-name": "airflow.providers.google.cloud.hooks.cloud_sql.CloudSQLDatabaseHook",
                "connection-type": "gcpcloudsqldb",
            },
            {
                "hook-class-name": "airflow.providers.google.cloud.hooks.bigquery.BigQueryHook",
                "connection-type": "gcpbigquery",
            },
            {
                "hook-class-name": "airflow.providers.google.cloud.hooks.compute_ssh.ComputeEngineSSHHook",
                "connection-type": "gcpssh",
            },
            {
                "hook-class-name": "airflow.providers.google.leveldb.hooks.leveldb.LevelDBHook",
                "connection-type": "leveldb",
            },
        ],
        "extra-links": [
            "airflow.providers.google.cloud.operators.bigquery.BigQueryConsoleLink",
            "airflow.providers.google.cloud.operators.bigquery.BigQueryConsoleIndexableLink",
            "airflow.providers.google.cloud.links.dataform.DataformRepositoryLink",
            "airflow.providers.google.cloud.links.dataform.DataformWorkspaceLink",
            "airflow.providers.google.cloud.links.dataform.DataformWorkflowInvocationLink",
            "airflow.providers.google.cloud.links.datafusion.DataFusionInstanceLink",
            "airflow.providers.google.cloud.links.datafusion.DataFusionPipelineLink",
            "airflow.providers.google.cloud.links.datafusion.DataFusionPipelinesLink",
            "airflow.providers.google.cloud.links.cloud_sql.CloudSQLInstanceLink",
            "airflow.providers.google.cloud.links.cloud_sql.CloudSQLInstanceDatabaseLink",
            "airflow.providers.google.cloud.links.dataplex.DataplexTaskLink",
            "airflow.providers.google.cloud.links.dataplex.DataplexTasksLink",
            "airflow.providers.google.cloud.links.dataplex.DataplexLakeLink",
            "airflow.providers.google.cloud.links.bigquery.BigQueryDatasetLink",
            "airflow.providers.google.cloud.links.bigquery.BigQueryTableLink",
            "airflow.providers.google.cloud.links.bigquery_dts.BigQueryDataTransferConfigLink",
            "airflow.providers.google.cloud.links.compute.ComputeInstanceDetailsLink",
            "airflow.providers.google.cloud.links.compute.ComputeInstanceTemplateDetailsLink",
            "airflow.providers.google.cloud.links.compute.ComputeInstanceGroupManagerDetailsLink",
            "airflow.providers.google.cloud.links.cloud_tasks.CloudTasksQueueLink",
            "airflow.providers.google.cloud.links.cloud_tasks.CloudTasksLink",
            "airflow.providers.google.cloud.links.datacatalog.DataCatalogEntryGroupLink",
            "airflow.providers.google.cloud.links.datacatalog.DataCatalogEntryLink",
            "airflow.providers.google.cloud.links.datacatalog.DataCatalogTagTemplateLink",
            "airflow.providers.google.cloud.links.dataproc.DataprocLink",
            "airflow.providers.google.cloud.links.dataproc.DataprocListLink",
            "airflow.providers.google.cloud.links.dataproc.DataprocClusterLink",
            "airflow.providers.google.cloud.links.dataproc.DataprocJobLink",
            "airflow.providers.google.cloud.links.dataproc.DataprocWorkflowLink",
            "airflow.providers.google.cloud.links.dataproc.DataprocWorkflowTemplateLink",
            "airflow.providers.google.cloud.links.dataproc.DataprocBatchLink",
            "airflow.providers.google.cloud.links.dataproc.DataprocBatchesListLink",
            "airflow.providers.google.cloud.operators.dataproc_metastore.DataprocMetastoreDetailedLink",
            "airflow.providers.google.cloud.operators.dataproc_metastore.DataprocMetastoreLink",
            "airflow.providers.google.cloud.links.dataprep.DataprepFlowLink",
            "airflow.providers.google.cloud.links.dataprep.DataprepJobGroupLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIModelLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIModelListLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIModelExportLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAITrainingLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAITrainingPipelinesLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIDatasetLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIDatasetListLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIHyperparameterTuningJobListLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIBatchPredictionJobLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIBatchPredictionJobListLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIEndpointLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIEndpointListLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIPipelineJobLink",
            "airflow.providers.google.cloud.links.vertex_ai.VertexAIPipelineJobListLink",
            "airflow.providers.google.cloud.links.workflows.WorkflowsWorkflowDetailsLink",
            "airflow.providers.google.cloud.links.workflows.WorkflowsListOfWorkflowsLink",
            "airflow.providers.google.cloud.links.workflows.WorkflowsExecutionLink",
            "airflow.providers.google.cloud.operators.cloud_composer.CloudComposerEnvironmentLink",
            "airflow.providers.google.cloud.operators.cloud_composer.CloudComposerEnvironmentsLink",
            "airflow.providers.google.cloud.links.dataflow.DataflowJobLink",
            "airflow.providers.google.cloud.links.dataflow.DataflowPipelineLink",
            "airflow.providers.google.cloud.links.datastore.CloudDatastoreImportExportLink",
            "airflow.providers.google.cloud.links.datastore.CloudDatastoreEntitiesLink",
            "airflow.providers.google.cloud.links.bigtable.BigtableInstanceLink",
            "airflow.providers.google.cloud.links.bigtable.BigtableClusterLink",
            "airflow.providers.google.cloud.links.bigtable.BigtableTablesLink",
            "airflow.providers.google.cloud.links.spanner.SpannerDatabaseLink",
            "airflow.providers.google.cloud.links.spanner.SpannerInstanceLink",
            "airflow.providers.google.cloud.links.stackdriver.StackdriverNotificationsLink",
            "airflow.providers.google.cloud.links.stackdriver.StackdriverPoliciesLink",
            "airflow.providers.google.cloud.links.kubernetes_engine.KubernetesEngineClusterLink",
            "airflow.providers.google.cloud.links.kubernetes_engine.KubernetesEnginePodLink",
            "airflow.providers.google.cloud.links.kubernetes_engine.KubernetesEngineJobLink",
            "airflow.providers.google.cloud.links.kubernetes_engine.KubernetesEngineWorkloadsLink",
            "airflow.providers.google.cloud.links.pubsub.PubSubSubscriptionLink",
            "airflow.providers.google.cloud.links.pubsub.PubSubTopicLink",
            "airflow.providers.google.cloud.links.cloud_memorystore.MemcachedInstanceDetailsLink",
            "airflow.providers.google.cloud.links.cloud_memorystore.MemcachedInstanceListLink",
            "airflow.providers.google.cloud.links.cloud_memorystore.RedisInstanceDetailsLink",
            "airflow.providers.google.cloud.links.cloud_memorystore.RedisInstanceListLink",
            "airflow.providers.google.cloud.links.cloud_build.CloudBuildLink",
            "airflow.providers.google.cloud.links.cloud_build.CloudBuildListLink",
            "airflow.providers.google.cloud.links.cloud_build.CloudBuildTriggersListLink",
            "airflow.providers.google.cloud.links.cloud_build.CloudBuildTriggerDetailsLink",
            "airflow.providers.google.cloud.links.automl.AutoMLDatasetLink",
            "airflow.providers.google.cloud.links.automl.AutoMLDatasetListLink",
            "airflow.providers.google.cloud.links.automl.AutoMLModelLink",
            "airflow.providers.google.cloud.links.automl.AutoMLModelTrainLink",
            "airflow.providers.google.cloud.links.automl.AutoMLModelPredictLink",
            "airflow.providers.google.cloud.links.life_sciences.LifeSciencesLink",
            "airflow.providers.google.cloud.links.cloud_functions.CloudFunctionsDetailsLink",
            "airflow.providers.google.cloud.links.cloud_functions.CloudFunctionsListLink",
            "airflow.providers.google.cloud.links.cloud_storage_transfer.CloudStorageTransferListLink",
            "airflow.providers.google.cloud.links.cloud_storage_transfer.CloudStorageTransferJobLink",
            "airflow.providers.google.cloud.links.cloud_storage_transfer.CloudStorageTransferDetailsLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPDeidentifyTemplatesListLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPDeidentifyTemplateDetailsLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPJobTriggersListLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPJobTriggerDetailsLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPJobsListLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPJobDetailsLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPInspectTemplatesListLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPInspectTemplateDetailsLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPInfoTypesListLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPInfoTypeDetailsLink",
            "airflow.providers.google.cloud.links.data_loss_prevention.CloudDLPPossibleInfoTypesListLink",
            "airflow.providers.google.cloud.links.mlengine.MLEngineModelLink",
            "airflow.providers.google.cloud.links.mlengine.MLEngineModelsListLink",
            "airflow.providers.google.cloud.links.mlengine.MLEngineJobDetailsLink",
            "airflow.providers.google.cloud.links.mlengine.MLEngineJobSListLink",
            "airflow.providers.google.cloud.links.mlengine.MLEngineModelVersionDetailsLink",
            "airflow.providers.google.common.links.storage.StorageLink",
            "airflow.providers.google.common.links.storage.FileDetailsLink",
            "airflow.providers.google.marketing_platform.links.analytics_admin.GoogleAnalyticsPropertyLink",
            "airflow.providers.google.cloud.links.translate.TranslationLegacyDatasetLink",
            "airflow.providers.google.cloud.links.translate.TranslationDatasetListLink",
            "airflow.providers.google.cloud.links.translate.TranslationLegacyModelLink",
            "airflow.providers.google.cloud.links.translate.TranslationLegacyModelTrainLink",
            "airflow.providers.google.cloud.links.translate.TranslationLegacyModelPredictLink",
        ],
        "secrets-backends": [
            "airflow.providers.google.cloud.secrets.secret_manager.CloudSecretManagerBackend"
        ],
        "auth-backends": ["airflow.providers.google.common.auth_backend.google_openid"],
        "logging": [
            "airflow.providers.google.cloud.log.gcs_task_handler.GCSTaskHandler",
            "airflow.providers.google.cloud.log.stackdriver_task_handler.StackdriverTaskHandler",
        ],
    }



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from typing import TYPE_CHECKING, ClassVar

from airflow.models import BaseOperatorLink, XCom

if TYPE_CHECKING:
    from airflow.models import BaseOperator
    from airflow.models.taskinstancekey import TaskInstanceKey


BASE_LINK = "https://console.cloud.google.com"


class BaseGoogleLink(BaseOperatorLink):
    """
    Base class for all Google links.

    :meta private:
    """

    name: ClassVar[str]
    key: ClassVar[str]
    format_str: ClassVar[str]

    def get_link(
        self,
        operator: BaseOperator,
        *,
        ti_key: TaskInstanceKey,
    ) -> str:
        conf = XCom.get_value(key=self.key, ti_key=ti_key)
        if not conf:
            return ""
        if self.format_str.startswith("http"):
            return self.format_str.format(**conf)
        return BASE_LINK + self.format_str.format(**conf)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""This module contains Google Cloud Transfer operators."""

from __future__ import annotations

from copy import deepcopy
from datetime import date, time
from typing import TYPE_CHECKING, Sequence

from airflow.exceptions import AirflowException
from airflow.providers.amazon.aws.hooks.base_aws import AwsBaseHook
from airflow.providers.google.cloud.hooks.cloud_storage_transfer_service import (
    ACCESS_KEY_ID,
    AWS_ACCESS_KEY,
    AWS_ROLE_ARN,
    AWS_S3_DATA_SOURCE,
    BUCKET_NAME,
    DAY,
    DESCRIPTION,
    GCS_DATA_SINK,
    GCS_DATA_SOURCE,
    HOURS,
    HTTP_DATA_SOURCE,
    MINUTES,
    MONTH,
    NAME,
    OBJECT_CONDITIONS,
    PATH,
    PROJECT_ID,
    SCHEDULE,
    SCHEDULE_END_DATE,
    SCHEDULE_START_DATE,
    SECONDS,
    SECRET_ACCESS_KEY,
    START_TIME_OF_DAY,
    STATUS,
    TRANSFER_OPTIONS,
    TRANSFER_SPEC,
    YEAR,
    CloudDataTransferServiceHook,
    GcpTransferJobsStatus,
)
from airflow.providers.google.cloud.links.cloud_storage_transfer import (
    CloudStorageTransferDetailsLink,
    CloudStorageTransferJobLink,
    CloudStorageTransferListLink,
)
from airflow.providers.google.cloud.operators.cloud_base import GoogleCloudBaseOperator
from airflow.providers.google.cloud.utils.helpers import normalize_directory_path
from airflow.providers.google.common.hooks.base_google import PROVIDE_PROJECT_ID

if TYPE_CHECKING:
    from airflow.utils.context import Context


class TransferJobPreprocessor:
    """Helper class for preprocess of transfer job body."""

    def __init__(
        self, body: dict, aws_conn_id: str | None = "aws_default", default_schedule: bool = False
    ) -> None:
        self.body = body
        self.aws_conn_id = aws_conn_id
        self.default_schedule = default_schedule

    def _inject_aws_credentials(self) -> None:
        if TRANSFER_SPEC not in self.body:
            return

        if AWS_S3_DATA_SOURCE not in self.body[TRANSFER_SPEC]:
            return

        if AWS_ROLE_ARN in self.body[TRANSFER_SPEC][AWS_S3_DATA_SOURCE]:
            return

        aws_hook = AwsBaseHook(self.aws_conn_id, resource_type="s3")
        aws_credentials = aws_hook.get_credentials()
        aws_access_key_id = aws_credentials.access_key  # type: ignore[attr-defined]
        aws_secret_access_key = aws_credentials.secret_key  # type: ignore[attr-defined]
        self.body[TRANSFER_SPEC][AWS_S3_DATA_SOURCE][AWS_ACCESS_KEY] = {
            ACCESS_KEY_ID: aws_access_key_id,
            SECRET_ACCESS_KEY: aws_secret_access_key,
        }

    def _reformat_date(self, field_key: str) -> None:
        schedule = self.body[SCHEDULE]
        if field_key not in schedule:
            return
        if isinstance(schedule[field_key], date):
            schedule[field_key] = self._convert_date_to_dict(schedule[field_key])

    def _reformat_time(self, field_key: str) -> None:
        schedule = self.body[SCHEDULE]
        if field_key not in schedule:
            return
        if isinstance(schedule[field_key], time):
            schedule[field_key] = self._convert_time_to_dict(schedule[field_key])

    def _reformat_schedule(self) -> None:
        if SCHEDULE not in self.body:
            if self.default_schedule:
                self.body[SCHEDULE] = {SCHEDULE_START_DATE: date.today(), SCHEDULE_END_DATE: date.today()}
            else:
                return
        self._reformat_date(SCHEDULE_START_DATE)
        self._reformat_date(SCHEDULE_END_DATE)
        self._reformat_time(START_TIME_OF_DAY)

    def process_body(self) -> dict:
        """
        Injects AWS credentials into body if needed and reformats schedule information.

        :return: Preprocessed body
        """
        self._inject_aws_credentials()
        self._reformat_schedule()
        return self.body

    @staticmethod
    def _convert_date_to_dict(field_date: date) -> dict:
        """Convert native python ``datetime.date`` object  to a format supported by the API."""
        return {DAY: field_date.day, MONTH: field_date.month, YEAR: field_date.year}

    @staticmethod
    def _convert_time_to_dict(time_object: time) -> dict:
        """Convert native python ``datetime.time`` object  to a format supported by the API."""
        return {HOURS: time_object.hour, MINUTES: time_object.minute, SECONDS: time_object.second}


class TransferJobValidator:
    """Helper class for validating transfer job body."""

    def __init__(self, body: dict) -> None:
        if not body:
            raise AirflowException("The required parameter 'body' is empty or None")

        self.body = body

    def _verify_data_source(self) -> None:
        is_gcs = GCS_DATA_SOURCE in self.body[TRANSFER_SPEC]
        is_aws_s3 = AWS_S3_DATA_SOURCE in self.body[TRANSFER_SPEC]
        is_http = HTTP_DATA_SOURCE in self.body[TRANSFER_SPEC]

        sources_count = sum([is_gcs, is_aws_s3, is_http])
        if sources_count > 1:
            raise AirflowException(
                "More than one data source detected. Please choose exactly one data source from: "
                "gcsDataSource, awsS3DataSource and httpDataSource."
            )

    def _restrict_aws_credentials(self) -> None:
        aws_transfer = AWS_S3_DATA_SOURCE in self.body[TRANSFER_SPEC]
        if aws_transfer and AWS_ACCESS_KEY in self.body[TRANSFER_SPEC][AWS_S3_DATA_SOURCE]:
            raise AirflowException(
                "AWS credentials detected inside the body parameter (awsAccessKey). This is not allowed, "
                "please use Airflow connections to store credentials."
            )

    def validate_body(self) -> None:
        """
        Validate the body.

        Checks if body specifies `transferSpec` if yes, then check if AWS credentials
        are passed correctly and no more than 1 data source was selected.

        :raises: AirflowException
        """
        if TRANSFER_SPEC in self.body:
            self._restrict_aws_credentials()
            self._verify_data_source()


class CloudDataTransferServiceCreateJobOperator(GoogleCloudBaseOperator):
    """
    Creates a transfer job that runs periodically.

    .. warning::

        This operator is NOT idempotent in the following cases:

        * `name` is not passed in body param
        * transfer job `name` has been soft deleted. In this case,
          each new task will receive a unique suffix

        If you run it many times, many transfer jobs will be created in the Google Cloud.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:CloudDataTransferServiceCreateJobOperator`

    :param body: (Required) The request body, as described in
        https://cloud.google.com/storage-transfer/docs/reference/rest/v1/transferJobs#TransferJob
        With three additional improvements:

        * dates can be given in the form :class:`datetime.date`
        * times can be given in the form :class:`datetime.time`
        * credentials to Amazon Web Service should be stored in the connection and indicated by the
          aws_conn_id parameter

    :param aws_conn_id: The connection ID used to retrieve credentials to
        Amazon Web Service.
    :param gcp_conn_id: The connection ID used to connect to Google Cloud.
    :param api_version: API version used (e.g. v1).
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    # [START gcp_transfer_job_create_template_fields]
    template_fields: Sequence[str] = (
        "body",
        "gcp_conn_id",
        "aws_conn_id",
        "google_impersonation_chain",
    )
    # [END gcp_transfer_job_create_template_fields]
    operator_extra_links = (CloudStorageTransferJobLink(),)

    def __init__(
        self,
        *,
        body: dict,
        aws_conn_id: str | None = "aws_default",
        gcp_conn_id: str = "google_cloud_default",
        api_version: str = "v1",
        project_id: str = PROVIDE_PROJECT_ID,
        google_impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.body = body
        if isinstance(self.body, dict):
            self.body = deepcopy(body)
        self.aws_conn_id = aws_conn_id
        self.gcp_conn_id = gcp_conn_id
        self.api_version = api_version
        self.project_id = project_id
        self.google_impersonation_chain = google_impersonation_chain
        self._validate_inputs()

    def _validate_inputs(self) -> None:
        TransferJobValidator(body=self.body).validate_body()

    def execute(self, context: Context) -> dict:
        TransferJobPreprocessor(body=self.body, aws_conn_id=self.aws_conn_id).process_body()
        hook = CloudDataTransferServiceHook(
            api_version=self.api_version,
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )
        result = hook.create_transfer_job(body=self.body)

        project_id = self.project_id or hook.project_id
        if project_id:
            CloudStorageTransferJobLink.persist(
                context=context,
                task_instance=self,
                project_id=project_id,
                job_name=result[NAME],
            )

        return result


class CloudDataTransferServiceUpdateJobOperator(GoogleCloudBaseOperator):
    """
    Updates a transfer job that runs periodically.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:CloudDataTransferServiceUpdateJobOperator`

    :param job_name: (Required) Name of the job to be updated
    :param body: (Required) The request body, as described in
        https://cloud.google.com/storage-transfer/docs/reference/rest/v1/transferJobs/patch#request-body
        With three additional improvements:

        * dates can be given in the form :class:`datetime.date`
        * times can be given in the form :class:`datetime.time`
        * credentials to Amazon Web Service should be stored in the connection and indicated by the
          aws_conn_id parameter

    :param aws_conn_id: The connection ID used to retrieve credentials to
        Amazon Web Service.
    :param gcp_conn_id: The connection ID used to connect to Google Cloud.
    :param api_version: API version used (e.g. v1).
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    # [START gcp_transfer_job_update_template_fields]
    template_fields: Sequence[str] = (
        "job_name",
        "body",
        "gcp_conn_id",
        "aws_conn_id",
        "google_impersonation_chain",
    )
    # [END gcp_transfer_job_update_template_fields]
    operator_extra_links = (CloudStorageTransferJobLink(),)

    def __init__(
        self,
        *,
        job_name: str,
        body: dict,
        aws_conn_id: str | None = "aws_default",
        gcp_conn_id: str = "google_cloud_default",
        api_version: str = "v1",
        project_id: str = PROVIDE_PROJECT_ID,
        google_impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.job_name = job_name
        self.body = body
        self.project_id = project_id
        self.gcp_conn_id = gcp_conn_id
        self.api_version = api_version
        self.aws_conn_id = aws_conn_id
        self.google_impersonation_chain = google_impersonation_chain
        self._validate_inputs()

    def _validate_inputs(self) -> None:
        TransferJobValidator(body=self.body).validate_body()
        if not self.job_name:
            raise AirflowException("The required parameter 'job_name' is empty or None")

    def execute(self, context: Context) -> dict:
        TransferJobPreprocessor(body=self.body, aws_conn_id=self.aws_conn_id).process_body()
        hook = CloudDataTransferServiceHook(
            api_version=self.api_version,
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )

        project_id = self.project_id or hook.project_id
        if project_id:
            CloudStorageTransferJobLink.persist(
                context=context,
                task_instance=self,
                project_id=project_id,
                job_name=self.job_name,
            )

        return hook.update_transfer_job(job_name=self.job_name, body=self.body)


class CloudDataTransferServiceDeleteJobOperator(GoogleCloudBaseOperator):
    """
    Delete a transfer job.

    This is a soft delete. After a transfer job is deleted, the job and all the transfer
    executions are subject to garbage collection. Transfer jobs become eligible for garbage
    collection 30 days after soft delete.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:CloudDataTransferServiceDeleteJobOperator`

    :param job_name: (Required) Name of the TRANSFER operation
    :param project_id: (Optional) the ID of the project that owns the Transfer
        Job. If set to None or missing, the default project_id from the Google Cloud
        connection is used.
    :param gcp_conn_id: The connection ID used to connect to Google Cloud.
    :param api_version: API version used (e.g. v1).
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    # [START gcp_transfer_job_delete_template_fields]
    template_fields: Sequence[str] = (
        "job_name",
        "project_id",
        "gcp_conn_id",
        "api_version",
        "google_impersonation_chain",
    )
    # [END gcp_transfer_job_delete_template_fields]

    def __init__(
        self,
        *,
        job_name: str,
        gcp_conn_id: str = "google_cloud_default",
        api_version: str = "v1",
        project_id: str = PROVIDE_PROJECT_ID,
        google_impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.job_name = job_name
        self.project_id = project_id
        self.gcp_conn_id = gcp_conn_id
        self.api_version = api_version
        self.google_impersonation_chain = google_impersonation_chain
        self._validate_inputs()

    def _validate_inputs(self) -> None:
        if not self.job_name:
            raise AirflowException("The required parameter 'job_name' is empty or None")

    def execute(self, context: Context) -> None:
        self._validate_inputs()
        hook = CloudDataTransferServiceHook(
            api_version=self.api_version,
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )
        hook.delete_transfer_job(job_name=self.job_name, project_id=self.project_id)


class CloudDataTransferServiceRunJobOperator(GoogleCloudBaseOperator):
    """
    Runs a transfer job.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:CloudDataTransferServiceRunJobOperator`

    :param job_name: (Required) Name of the job to be run
    :param project_id: (Optional) the ID of the project that owns the Transfer
        Job. If set to None or missing, the default project_id from the Google Cloud
        connection is used.
    :param gcp_conn_id: The connection ID used to connect to Google Cloud.
    :param api_version: API version used (e.g. v1).
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    # [START gcp_transfer_job_run_template_fields]
    template_fields: Sequence[str] = (
        "job_name",
        "project_id",
        "gcp_conn_id",
        "api_version",
        "google_impersonation_chain",
    )
    # [END gcp_transfer_job_run_template_fields]
    operator_extra_links = (CloudStorageTransferJobLink(),)

    def __init__(
        self,
        *,
        job_name: str,
        gcp_conn_id: str = "google_cloud_default",
        api_version: str = "v1",
        project_id: str = PROVIDE_PROJECT_ID,
        google_impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.job_name = job_name
        self.project_id = project_id
        self.gcp_conn_id = gcp_conn_id
        self.api_version = api_version
        self.google_impersonation_chain = google_impersonation_chain

    def _validate_inputs(self) -> None:
        if not self.job_name:
            raise AirflowException("The required parameter 'job_name' is empty or None")

    def execute(self, context: Context) -> dict:
        self._validate_inputs()
        hook = CloudDataTransferServiceHook(
            api_version=self.api_version,
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )

        project_id = self.project_id or hook.project_id
        if project_id:
            CloudStorageTransferJobLink.persist(
                context=context,
                task_instance=self,
                project_id=project_id,
                job_name=self.job_name,
            )

        return hook.run_transfer_job(job_name=self.job_name, project_id=project_id)


class CloudDataTransferServiceGetOperationOperator(GoogleCloudBaseOperator):
    """
    Gets the latest state of a long-running operation in Google Storage Transfer Service.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:CloudDataTransferServiceGetOperationOperator`

    :param operation_name: (Required) Name of the transfer operation.
    :param gcp_conn_id: The connection ID used to connect to Google
        Cloud Platform.
    :param api_version: API version used (e.g. v1).
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    # [START gcp_transfer_operation_get_template_fields]
    template_fields: Sequence[str] = (
        "operation_name",
        "gcp_conn_id",
        "google_impersonation_chain",
    )
    # [END gcp_transfer_operation_get_template_fields]
    operator_extra_links = (CloudStorageTransferDetailsLink(),)

    def __init__(
        self,
        *,
        project_id: str = PROVIDE_PROJECT_ID,
        operation_name: str,
        gcp_conn_id: str = "google_cloud_default",
        api_version: str = "v1",
        google_impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.operation_name = operation_name
        self.project_id = project_id
        self.gcp_conn_id = gcp_conn_id
        self.api_version = api_version
        self.google_impersonation_chain = google_impersonation_chain
        self._validate_inputs()

    def _validate_inputs(self) -> None:
        if not self.operation_name:
            raise AirflowException("The required parameter 'operation_name' is empty or None")

    def execute(self, context: Context) -> dict:
        hook = CloudDataTransferServiceHook(
            api_version=self.api_version,
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )
        operation = hook.get_transfer_operation(operation_name=self.operation_name)

        project_id = self.project_id or hook.project_id
        if project_id:
            CloudStorageTransferDetailsLink.persist(
                context=context,
                task_instance=self,
                project_id=project_id,
                operation_name=self.operation_name,
            )

        return operation


class CloudDataTransferServiceListOperationsOperator(GoogleCloudBaseOperator):
    """
    Lists long-running operations in Google Storage Transfer Service that match the specified filter.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:CloudDataTransferServiceListOperationsOperator`

    :param request_filter: (Required) A request filter, as described in
            https://cloud.google.com/storage-transfer/docs/reference/rest/v1/transferJobs/list#body.QUERY_PARAMETERS.filter
    :param gcp_conn_id: The connection ID used to connect to Google
        Cloud Platform.
    :param api_version: API version used (e.g. v1).
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    # [START gcp_transfer_operations_list_template_fields]
    template_fields: Sequence[str] = (
        "request_filter",
        "gcp_conn_id",
        "google_impersonation_chain",
    )
    # [END gcp_transfer_operations_list_template_fields]
    operator_extra_links = (CloudStorageTransferListLink(),)

    def __init__(
        self,
        request_filter: dict,
        project_id: str = PROVIDE_PROJECT_ID,
        gcp_conn_id: str = "google_cloud_default",
        api_version: str = "v1",
        google_impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.request_filter = request_filter
        self.project_id = project_id
        self.gcp_conn_id = gcp_conn_id
        self.api_version = api_version
        self.google_impersonation_chain = google_impersonation_chain

    @property
    def filter(self) -> dict | None:
        """Alias for ``request_filter``, used for compatibility."""
        return self.request_filter

    def execute(self, context: Context) -> list[dict]:
        if not self.request_filter:
            raise AirflowException("The required parameter 'request_filter' is empty or None")

        hook = CloudDataTransferServiceHook(
            api_version=self.api_version,
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )
        operations_list = hook.list_transfer_operations(request_filter=self.request_filter)
        self.log.info(operations_list)

        project_id = self.project_id or hook.project_id
        if project_id:
            CloudStorageTransferListLink.persist(
                context=context,
                task_instance=self,
                project_id=project_id,
            )

        return operations_list


class CloudDataTransferServicePauseOperationOperator(GoogleCloudBaseOperator):
    """
    Pauses a transfer operation in Google Storage Transfer Service.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:CloudDataTransferServicePauseOperationOperator`

    :param operation_name: (Required) Name of the transfer operation.
    :param gcp_conn_id: The connection ID used to connect to Google Cloud.
    :param api_version:  API version used (e.g. v1).
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    # [START gcp_transfer_operation_pause_template_fields]
    template_fields: Sequence[str] = (
        "operation_name",
        "gcp_conn_id",
        "api_version",
        "google_impersonation_chain",
    )
    # [END gcp_transfer_operation_pause_template_fields]

    def __init__(
        self,
        *,
        operation_name: str,
        gcp_conn_id: str = "google_cloud_default",
        api_version: str = "v1",
        google_impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.operation_name = operation_name
        self.gcp_conn_id = gcp_conn_id
        self.api_version = api_version
        self.google_impersonation_chain = google_impersonation_chain
        self._validate_inputs()

    def _validate_inputs(self) -> None:
        if not self.operation_name:
            raise AirflowException("The required parameter 'operation_name' is empty or None")

    def execute(self, context: Context) -> None:
        hook = CloudDataTransferServiceHook(
            api_version=self.api_version,
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )
        hook.pause_transfer_operation(operation_name=self.operation_name)


class CloudDataTransferServiceResumeOperationOperator(GoogleCloudBaseOperator):
    """
    Resumes a transfer operation in Google Storage Transfer Service.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:CloudDataTransferServiceResumeOperationOperator`

    :param operation_name: (Required) Name of the transfer operation.
    :param gcp_conn_id: The connection ID used to connect to Google Cloud.
    :param api_version: API version used (e.g. v1).
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    # [START gcp_transfer_operation_resume_template_fields]
    template_fields: Sequence[str] = (
        "operation_name",
        "gcp_conn_id",
        "api_version",
        "google_impersonation_chain",
    )
    # [END gcp_transfer_operation_resume_template_fields]

    def __init__(
        self,
        *,
        operation_name: str,
        gcp_conn_id: str = "google_cloud_default",
        api_version: str = "v1",
        google_impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        self.operation_name = operation_name
        self.gcp_conn_id = gcp_conn_id
        self.api_version = api_version
        self.google_impersonation_chain = google_impersonation_chain
        self._validate_inputs()
        super().__init__(**kwargs)

    def _validate_inputs(self) -> None:
        if not self.operation_name:
            raise AirflowException("The required parameter 'operation_name' is empty or None")

    def execute(self, context: Context) -> None:
        hook = CloudDataTransferServiceHook(
            api_version=self.api_version,
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )
        hook.resume_transfer_operation(operation_name=self.operation_name)


class CloudDataTransferServiceCancelOperationOperator(GoogleCloudBaseOperator):
    """
    Cancels a transfer operation in Google Storage Transfer Service.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:CloudDataTransferServiceCancelOperationOperator`

    :param operation_name: (Required) Name of the transfer operation.
    :param api_version: API version used (e.g. v1).
    :param gcp_conn_id: The connection ID used to connect to Google
        Cloud Platform.
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    # [START gcp_transfer_operation_cancel_template_fields]
    template_fields: Sequence[str] = (
        "operation_name",
        "gcp_conn_id",
        "api_version",
        "google_impersonation_chain",
    )
    # [END gcp_transfer_operation_cancel_template_fields]

    def __init__(
        self,
        *,
        operation_name: str,
        gcp_conn_id: str = "google_cloud_default",
        api_version: str = "v1",
        google_impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.operation_name = operation_name
        self.api_version = api_version
        self.gcp_conn_id = gcp_conn_id
        self.google_impersonation_chain = google_impersonation_chain
        self._validate_inputs()

    def _validate_inputs(self) -> None:
        if not self.operation_name:
            raise AirflowException("The required parameter 'operation_name' is empty or None")

    def execute(self, context: Context) -> None:
        hook = CloudDataTransferServiceHook(
            api_version=self.api_version,
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )
        hook.cancel_transfer_operation(operation_name=self.operation_name)


class CloudDataTransferServiceS3ToGCSOperator(GoogleCloudBaseOperator):
    """
    Sync an S3 bucket with a Google Cloud Storage bucket using the Google Cloud Storage Transfer Service.

    .. warning::

        This operator is NOT idempotent. If you run it many times, many transfer
        jobs will be created in the Google Cloud.

    **Example**:

    .. code-block:: python

       s3_to_gcs_transfer_op = S3ToGoogleCloudStorageTransferOperator(
           task_id="s3_to_gcs_transfer_example",
           s3_bucket="my-s3-bucket",
           project_id="my-gcp-project",
           gcs_bucket="my-gcs-bucket",
           dag=my_dag,
       )

    :param s3_bucket: The S3 bucket where to find the objects. (templated)
    :param gcs_bucket: The destination Google Cloud Storage bucket
        where you want to store the files. (templated)
    :param s3_path: Optional root path where the source objects are. (templated)
    :param gcs_path: Optional root path for transferred objects. (templated)
    :param project_id: Optional ID of the Google Cloud Console project that
        owns the job
    :param aws_conn_id: The source S3 connection
    :param gcp_conn_id: The destination connection ID to use
        when connecting to Google Cloud Storage.
    :param description: Optional transfer service job description
    :param schedule: Optional transfer service schedule;
        If not set, run transfer job once as soon as the operator runs
        The format is described
        https://cloud.google.com/storage-transfer/docs/reference/rest/v1/transferJobs.
        With two additional improvements:

        * dates they can be passed as :class:`datetime.date`
        * times they can be passed as :class:`datetime.time`

    :param object_conditions: Optional transfer service object conditions; see
        https://cloud.google.com/storage-transfer/docs/reference/rest/v1/TransferSpec
    :param transfer_options: Optional transfer service transfer options; see
        https://cloud.google.com/storage-transfer/docs/reference/rest/v1/TransferSpec
    :param wait: Wait for transfer to finish. It must be set to True, if
        'delete_job_after_completion' is set to True.
    :param timeout: Time to wait for the operation to end in seconds. Defaults to 60 seconds if not specified.
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    :param delete_job_after_completion: If True, delete the job after complete.
        If set to True, 'wait' must be set to True.
    :param aws_role_arn: Optional AWS role ARN for workload identity federation. This will
        override the `aws_conn_id` for authentication between GCP and AWS; see
        https://cloud.google.com/storage-transfer/docs/reference/rest/v1/TransferSpec#AwsS3Data
    """

    template_fields: Sequence[str] = (
        "gcp_conn_id",
        "s3_bucket",
        "gcs_bucket",
        "s3_path",
        "gcs_path",
        "description",
        "object_conditions",
        "google_impersonation_chain",
        "aws_role_arn",
    )
    ui_color = "#e09411"

    def __init__(
        self,
        *,
        s3_bucket: str,
        gcs_bucket: str,
        s3_path: str | None = None,
        gcs_path: str | None = None,
        project_id: str = PROVIDE_PROJECT_ID,
        aws_conn_id: str | None = "aws_default",
        gcp_conn_id: str = "google_cloud_default",
        description: str | None = None,
        schedule: dict | None = None,
        object_conditions: dict | None = None,
        transfer_options: dict | None = None,
        wait: bool = True,
        timeout: float | None = None,
        google_impersonation_chain: str | Sequence[str] | None = None,
        delete_job_after_completion: bool = False,
        aws_role_arn: str | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.s3_bucket = s3_bucket
        self.gcs_bucket = gcs_bucket
        self.s3_path = s3_path
        self.gcs_path = gcs_path
        self.project_id = project_id
        self.aws_conn_id = aws_conn_id
        self.gcp_conn_id = gcp_conn_id
        self.description = description
        self.schedule = schedule
        self.object_conditions = object_conditions
        self.transfer_options = transfer_options
        self.wait = wait
        self.timeout = timeout
        self.google_impersonation_chain = google_impersonation_chain
        self.delete_job_after_completion = delete_job_after_completion
        self.aws_role_arn = aws_role_arn
        self._validate_inputs()

    def _validate_inputs(self) -> None:
        if self.delete_job_after_completion and not self.wait:
            raise AirflowException("If 'delete_job_after_completion' is True, then 'wait' must also be True.")

    def execute(self, context: Context) -> None:
        hook = CloudDataTransferServiceHook(
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )
        body = self._create_body()

        TransferJobPreprocessor(body=body, aws_conn_id=self.aws_conn_id, default_schedule=True).process_body()

        job = hook.create_transfer_job(body=body)

        if self.wait:
            hook.wait_for_transfer_job(job, timeout=self.timeout)
            if self.delete_job_after_completion:
                hook.delete_transfer_job(job_name=job[NAME], project_id=self.project_id)

    def _create_body(self) -> dict:
        body = {
            DESCRIPTION: self.description,
            STATUS: GcpTransferJobsStatus.ENABLED,
            TRANSFER_SPEC: {
                AWS_S3_DATA_SOURCE: {
                    BUCKET_NAME: self.s3_bucket,
                    PATH: normalize_directory_path(self.s3_path),
                },
                GCS_DATA_SINK: {
                    BUCKET_NAME: self.gcs_bucket,
                    PATH: normalize_directory_path(self.gcs_path),
                },
            },
        }

        if self.project_id is not None:
            body[PROJECT_ID] = self.project_id

        if self.schedule is not None:
            body[SCHEDULE] = self.schedule

        if self.object_conditions is not None:
            body[TRANSFER_SPEC][OBJECT_CONDITIONS] = self.object_conditions  # type: ignore[index]

        if self.transfer_options is not None:
            body[TRANSFER_SPEC][TRANSFER_OPTIONS] = self.transfer_options  # type: ignore[index]

        if self.aws_role_arn is not None:
            body[TRANSFER_SPEC][AWS_S3_DATA_SOURCE][AWS_ROLE_ARN] = self.aws_role_arn  # type: ignore[index]

        return body


class CloudDataTransferServiceGCSToGCSOperator(GoogleCloudBaseOperator):
    """
    Copies objects from a bucket to another using the Google Cloud Storage Transfer Service.

    .. warning::

        This operator is NOT idempotent. If you run it many times, many transfer
        jobs will be created in the Google Cloud.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:CloudDataTransferServiceGCSToGCSOperator`

    **Example**:

    .. code-block:: python

       gcs_to_gcs_transfer_op = GoogleCloudStorageToGoogleCloudStorageTransferOperator(
           task_id="gcs_to_gcs_transfer_example",
           source_bucket="my-source-bucket",
           destination_bucket="my-destination-bucket",
           project_id="my-gcp-project",
           dag=my_dag,
       )

    :param source_bucket: The source Google Cloud Storage bucket where the
         object is. (templated)
    :param destination_bucket: The destination Google Cloud Storage bucket
        where the object should be. (templated)
    :param source_path: Optional root path where the source objects are. (templated)
    :param destination_path: Optional root path for transferred objects. (templated)
    :param project_id: The ID of the Google Cloud Console project that
        owns the job
    :param gcp_conn_id: Optional connection ID to use when connecting to Google Cloud
        Storage.
    :param description: Optional transfer service job description
    :param schedule: Optional transfer service schedule;
        If not set, run transfer job once as soon as the operator runs
        See:
        https://cloud.google.com/storage-transfer/docs/reference/rest/v1/transferJobs.
        With two additional improvements:

        * dates they can be passed as :class:`datetime.date`
        * times they can be passed as :class:`datetime.time`

    :param object_conditions: Optional transfer service object conditions; see
        https://cloud.google.com/storage-transfer/docs/reference/rest/v1/TransferSpec#ObjectConditions
    :param transfer_options: Optional transfer service transfer options; see
        https://cloud.google.com/storage-transfer/docs/reference/rest/v1/TransferSpec#TransferOptions
    :param wait: Wait for transfer to finish. It must be set to True, if
        'delete_job_after_completion' is set to True.
    :param timeout: Time to wait for the operation to end in seconds. Defaults to 60 seconds if not specified.
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    :param delete_job_after_completion: If True, delete the job after complete.
        If set to True, 'wait' must be set to True.
    """

    # [START gcp_transfer_gcs_to_gcs_template_fields]
    template_fields: Sequence[str] = (
        "gcp_conn_id",
        "source_bucket",
        "destination_bucket",
        "source_path",
        "destination_path",
        "description",
        "object_conditions",
        "google_impersonation_chain",
    )
    # [END gcp_transfer_gcs_to_gcs_template_fields]

    ui_color = "#e09411"

    def __init__(
        self,
        *,
        source_bucket: str,
        destination_bucket: str,
        source_path: str | None = None,
        destination_path: str | None = None,
        project_id: str = PROVIDE_PROJECT_ID,
        gcp_conn_id: str = "google_cloud_default",
        description: str | None = None,
        schedule: dict | None = None,
        object_conditions: dict | None = None,
        transfer_options: dict | None = None,
        wait: bool = True,
        timeout: float | None = None,
        google_impersonation_chain: str | Sequence[str] | None = None,
        delete_job_after_completion: bool = False,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.source_bucket = source_bucket
        self.destination_bucket = destination_bucket
        self.source_path = source_path
        self.destination_path = destination_path
        self.project_id = project_id
        self.gcp_conn_id = gcp_conn_id
        self.description = description
        self.schedule = schedule
        self.object_conditions = object_conditions
        self.transfer_options = transfer_options
        self.wait = wait
        self.timeout = timeout
        self.google_impersonation_chain = google_impersonation_chain
        self.delete_job_after_completion = delete_job_after_completion
        self._validate_inputs()

    def _validate_inputs(self) -> None:
        if self.delete_job_after_completion and not self.wait:
            raise AirflowException("If 'delete_job_after_completion' is True, then 'wait' must also be True.")

    def execute(self, context: Context) -> None:
        hook = CloudDataTransferServiceHook(
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.google_impersonation_chain,
        )

        body = self._create_body()

        TransferJobPreprocessor(body=body, default_schedule=True).process_body()

        job = hook.create_transfer_job(body=body)

        if self.wait:
            hook.wait_for_transfer_job(job, timeout=self.timeout)
            if self.delete_job_after_completion:
                hook.delete_transfer_job(job_name=job[NAME], project_id=self.project_id)

    def _create_body(self) -> dict:
        body = {
            DESCRIPTION: self.description,
            STATUS: GcpTransferJobsStatus.ENABLED,
            TRANSFER_SPEC: {
                GCS_DATA_SOURCE: {
                    BUCKET_NAME: self.source_bucket,
                    PATH: normalize_directory_path(self.source_path),
                },
                GCS_DATA_SINK: {
                    BUCKET_NAME: self.destination_bucket,
                    PATH: normalize_directory_path(self.destination_path),
                },
            },
        }

        if self.project_id is not None:
            body[PROJECT_ID] = self.project_id

        if self.schedule is not None:
            body[SCHEDULE] = self.schedule

        if self.object_conditions is not None:
            body[TRANSFER_SPEC][OBJECT_CONDITIONS] = self.object_conditions  # type: ignore[index]

        if self.transfer_options is not None:
            body[TRANSFER_SPEC][TRANSFER_OPTIONS] = self.transfer_options  # type: ignore[index]

        return body



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""This module contains Google BigQuery Data Transfer links."""

from __future__ import annotations

from typing import TYPE_CHECKING

from airflow.providers.google.cloud.links.base import BaseGoogleLink

if TYPE_CHECKING:
    from airflow.models import BaseOperator
    from airflow.utils.context import Context

BIGQUERY_BASE_LINK = "/bigquery/transfers"
BIGQUERY_DTS_LINK = BIGQUERY_BASE_LINK + "/locations/{region}/configs/{config_id}/runs?project={project_id}"


class BigQueryDataTransferConfigLink(BaseGoogleLink):
    """Helper class for constructing BigQuery Data Transfer Config Link."""

    name = "BigQuery Data Transfer Config"
    key = "bigquery_dts_config"
    format_str = BIGQUERY_DTS_LINK

    @staticmethod
    def persist(
        context: Context,
        task_instance: BaseOperator,
        region: str,
        config_id: str,
        project_id: str,
    ):
        task_instance.xcom_push(
            context,
            key=BigQueryDataTransferConfigLink.key,
            value={"project_id": project_id, "region": region, "config_id": config_id},
        )



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from __future__ import annotations

import json
from enum import Enum
from typing import Mapping

from airflow.providers.google.cloud.operators.dataform import (
    DataformInstallNpmPackagesOperator,
    DataformMakeDirectoryOperator,
    DataformWriteFileOperator,
)


class DataformLocations(str, Enum):
    """Enum for storing available locations for resources in Dataform."""

    US = "US"
    EUROPE = "EU"


def make_initialization_workspace_flow(
    project_id: str,
    region: str,
    repository_id: str,
    workspace_id: str,
    dataform_schema_name: str = "dataform",
    package_name: str | None = None,
    without_installation: bool = False,
) -> tuple:
    """
    Create flow which simulates the initialization of the default project.

    :param project_id: Required. The ID of the Google Cloud project where workspace located.
    :param region: Required. The ID of the Google Cloud region where workspace located.
    :param repository_id: Required. The ID of the Dataform repository where workspace located.
    :param workspace_id: Required. The ID of the Dataform workspace which requires initialization.
    :param dataform_schema_name: Name of the schema.
    :param package_name: Name of the package. If value is not provided then workspace_id will be used.
    :param without_installation: Defines should installation of npm packages be added to flow.
    """
    make_definitions_directory = DataformMakeDirectoryOperator(
        task_id="make-definitions-directory",
        project_id=project_id,
        region=region,
        repository_id=repository_id,
        workspace_id=workspace_id,
        directory_path="definitions",
    )

    first_view_content = b"""
        -- This is an example SQLX file to help you learn the basics of Dataform.
        -- Visit https://cloud.google.com/dataform/docs/how-to for more information on how to configure
        --   your SQL workflow.

        -- You can delete this file, then commit and push your changes to your repository when you are ready.

        -- Config blocks allow you to configure, document, and test your data assets.
        config {
          type: "view", // Creates a view in BigQuery. Try changing to "table" instead.
          columns: {
            test: "A description for the test column", // Column descriptions are pushed to BigQuery.
          }
        }

        -- The rest of a SQLX file contains your SELECT statement used to create the table.

        SELECT 1 as test
    """
    make_first_view_file = DataformWriteFileOperator(
        task_id="write-first-view",
        project_id=project_id,
        region=region,
        repository_id=repository_id,
        workspace_id=workspace_id,
        filepath="definitions/first_view.sqlx",
        contents=first_view_content,
    )

    second_view_content = b"""
        config { type: "view" }

        -- Use the ref() function to manage dependencies.
        -- Learn more about ref() and other built in functions
        --  here: https://cloud.google.com/dataform/docs/dataform-core

        SELECT test from ${ref("first_view")}
    """
    make_second_view_file = DataformWriteFileOperator(
        task_id="write-second-view",
        project_id=project_id,
        region=region,
        repository_id=repository_id,
        workspace_id=workspace_id,
        filepath="definitions/second_view.sqlx",
        contents=second_view_content,
    )

    make_includes_directory = DataformMakeDirectoryOperator(
        task_id="make-includes-directory",
        project_id=project_id,
        region=region,
        repository_id=repository_id,
        workspace_id=workspace_id,
        directory_path="includes",
    )

    gitignore_contents = b"""
    node_modules/
    """
    make_gitignore_file = DataformWriteFileOperator(
        task_id="write-gitignore-file",
        project_id=project_id,
        region=region,
        repository_id=repository_id,
        workspace_id=workspace_id,
        filepath=".gitignore",
        contents=gitignore_contents,
    )

    default_location: str = define_default_location(region).value
    dataform_config_content = json.dumps(
        {
            "defaultSchema": dataform_schema_name,
            "assertionSchema": "dataform_assertions",
            "warehouse": "bigquery",
            "defaultDatabase": project_id,
            "defaultLocation": default_location,
        },
        indent=4,
    ).encode()
    make_dataform_config_file = DataformWriteFileOperator(
        task_id="write-dataform-config-file",
        project_id=project_id,
        region=region,
        repository_id=repository_id,
        workspace_id=workspace_id,
        filepath="dataform.json",
        contents=dataform_config_content,
    )

    package_name = package_name or workspace_id
    package_json_content = json.dumps(
        {
            "name": package_name,
            "dependencies": {
                "@dataform/core": "2.0.1",
            },
        },
        indent=4,
    ).encode()
    make_package_json_file = DataformWriteFileOperator(
        task_id="write-package-json",
        project_id=project_id,
        region=region,
        repository_id=repository_id,
        workspace_id=workspace_id,
        filepath="package.json",
        contents=package_json_content,
    )

    (
        make_definitions_directory
        >> make_first_view_file
        >> make_second_view_file
        >> make_gitignore_file
        >> make_dataform_config_file
        >> make_package_json_file
    )

    if without_installation:
        make_package_json_file >> make_includes_directory
    else:
        install_npm_packages = DataformInstallNpmPackagesOperator(
            task_id="install-npm-packages",
            project_id=project_id,
            region=region,
            repository_id=repository_id,
            workspace_id=workspace_id,
        )
        make_package_json_file >> install_npm_packages >> make_includes_directory

    return make_definitions_directory, make_includes_directory


def define_default_location(region: str) -> DataformLocations:
    if "us" in region:
        return DataformLocations.US
    elif "europe" in region:
        return DataformLocations.EUROPE

    regions_mapping: Mapping[str, DataformLocations] = {}

    return regions_mapping[region]



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""This module contains Google Data Pipelines operators."""

from __future__ import annotations

from deprecated import deprecated

from airflow.exceptions import AirflowProviderDeprecationWarning
from airflow.providers.google.cloud.hooks.dataflow import DEFAULT_DATAFLOW_LOCATION
from airflow.providers.google.cloud.operators.dataflow import (
    DataflowCreatePipelineOperator,
    DataflowRunPipelineOperator,
)
from airflow.providers.google.common.hooks.base_google import PROVIDE_PROJECT_ID


@deprecated(
    reason="This operator is deprecated and will be removed after 01.12.2024. "
    "Please use `DataflowCreatePipelineOperator`.",
    category=AirflowProviderDeprecationWarning,
)
class CreateDataPipelineOperator(DataflowCreatePipelineOperator):
    """Creates a new Data Pipelines instance from the Data Pipelines API."""


@deprecated(
    reason="This operator is deprecated and will be removed after 01.12.2024. "
    "Please use `DataflowRunPipelineOperator`.",
    category=AirflowProviderDeprecationWarning,
)
class RunDataPipelineOperator(DataflowRunPipelineOperator):
    """Runs a Data Pipelines Instance using the Data Pipelines API."""

    def __init__(
        self,
        data_pipeline_name: str,
        project_id: str = PROVIDE_PROJECT_ID,
        location: str = DEFAULT_DATAFLOW_LOCATION,
        gcp_conn_id: str = "google_cloud_default",
        **kwargs,
    ) -> None:
        super().__init__(
            pipeline_name=data_pipeline_name,
            project_id=project_id,
            location=location,
            gcp_conn_id=gcp_conn_id,
            **kwargs,
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""This module contains Google Spanner links."""

from __future__ import annotations

from typing import TYPE_CHECKING

from airflow.providers.google.cloud.links.base import BaseGoogleLink

if TYPE_CHECKING:
    from airflow.models import BaseOperator
    from airflow.utils.context import Context

SPANNER_BASE_LINK = "/spanner/instances"
SPANNER_INSTANCE_LINK = SPANNER_BASE_LINK + "/{instance_id}/details/databases?project={project_id}"
SPANNER_DATABASE_LINK = (
    SPANNER_BASE_LINK + "/{instance_id}/databases/{database_id}/details/tables?project={project_id}"
)


class SpannerInstanceLink(BaseGoogleLink):
    """Helper class for constructing Spanner Instance Link."""

    name = "Spanner Instance"
    key = "spanner_instance"
    format_str = SPANNER_INSTANCE_LINK

    @staticmethod
    def persist(
        context: Context,
        task_instance: BaseOperator,
        instance_id: str,
        project_id: str | None,
    ):
        task_instance.xcom_push(
            context,
            key=SpannerInstanceLink.key,
            value={"instance_id": instance_id, "project_id": project_id},
        )


class SpannerDatabaseLink(BaseGoogleLink):
    """Helper class for constructing Spanner Database Link."""

    name = "Spanner Database"
    key = "spanner_database"
    format_str = SPANNER_DATABASE_LINK

    @staticmethod
    def persist(
        context: Context,
        task_instance: BaseOperator,
        instance_id: str,
        database_id: str,
        project_id: str | None,
    ):
        task_instance.xcom_push(
            context,
            key=SpannerDatabaseLink.key,
            value={"instance_id": instance_id, "database_id": database_id, "project_id": project_id},
        )
